/*************************************
 * RDesktop-GTK                      *
 *                                   *
 * A GTK front-end to rdesktop       *
 *                                   *
 * by JJ Foote, 01-15-2004           *
 *                                   *
 * Licensed under the GNU GPL.       *
 * NO WARRANTY, not even the implied *
 * warranty of merchantability or    *
 * fitness for a particular purpose. *
 *************************************/

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>

//#define RCFILE ".rdesktoprc"
//#define RDESKTOP "/usr/bin/X11/rdesktop"

// Global GTK widgets
GtkWidget *window, *hostname, *username, *password, *resolution;

// Prototypes
int save_settings();
int load_settings();
gint death();
gint system_call(gpointer data);
gint callback(GtkWidget *widget, gpointer data);

int save_settings()
{
	FILE* fp;
	char *temp;
	char filename[100];

	strcpy(filename, getenv("HOME"));
	strcat(filename, "/");
	strcat(filename, RCFILE);

	fp = fopen(filename, "w");
	if(fp == NULL)
		return 1;

	fprintf(fp, "hostname=%s\n", gtk_entry_get_text(GTK_ENTRY(hostname)));
	fprintf(fp, "username=%s\n", gtk_entry_get_text(GTK_ENTRY(username)));
	fprintf(fp, "password=%s\n", gtk_entry_get_text(GTK_ENTRY(password)));
	fprintf(fp, "resolution=%s\n", gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(resolution)->entry)));

	fclose(fp);

	return 0;
}

int load_settings()
{
	FILE* fp;
	char *temp, *ptr;
	char line[100], tmp[100], filename[100];

	strcpy(filename, getenv("HOME"));
	strcat(filename, "/");
	strcat(filename, RCFILE);

	fp = fopen(filename, "r");
	if(fp == NULL)
		return 1;

	while(!feof(fp)) {
		fgets(line, 100, fp);

		ptr = strstr(line, "username=");
		if(ptr != NULL) {
			strncpy(tmp, (ptr + 9), strlen(line) - 10);
			tmp[strlen(line) - 10] = '\0';
			gtk_entry_set_text(GTK_ENTRY(username), tmp);
		}

		ptr = strstr(line, "password=");
		if(ptr != NULL) {
			strncpy(tmp, (ptr + 9), strlen(line) - 10);
			tmp[strlen(line) - 10] = '\0';
			gtk_entry_set_text(GTK_ENTRY(password), tmp);
		}

		ptr = strstr(line, "hostname=");
		if(ptr != NULL) {
			strncpy(tmp, (ptr + 9), strlen(line) - 10);
			tmp[strlen(line) - 10] = '\0';
			gtk_entry_set_text(GTK_ENTRY(hostname), tmp);
		}

		ptr = strstr(line, "resolution=");
		if(ptr != NULL) {
			strncpy(tmp, (ptr + 11), strlen(line) - 12);
			tmp[strlen(line) - 12] = '\0';
			gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(resolution)->entry), tmp);
		}
	}

	fclose(fp);

	return 0;
}

gint death()
{
	save_settings();
	gtk_main_quit();
	return FALSE;
}

gint system_call(gpointer data)
{
	system((char*) data);
	return FALSE;
}

gint callback(GtkWidget *widget, gpointer data)
{
	static char command_line[512];

	strcpy(command_line, RDESKTOP);
	strcat(command_line, " -u");
	strcat(command_line, gtk_entry_get_text(GTK_ENTRY(username)));
	strcat(command_line, " -p");
	strcat(command_line, gtk_entry_get_text(GTK_ENTRY(password)));
	strcat(command_line, " -g");
	strcat(command_line, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(resolution)->entry)));
	strcat(command_line, " ");
	strcat(command_line, gtk_entry_get_text(GTK_ENTRY(hostname)));

	printf("Connecting to %s...\n", gtk_entry_get_text(GTK_ENTRY(hostname)));
	//printf("Command line: %s\n", command_line);

	system(command_line);
	//gtk_timeout_add(100, (GtkFunction) system_call, (gpointer) command_line);

	return TRUE;
}

int main(int argc, char **argv)
{
	GtkWidget *table, *button, *label;
	GList *resolution_choices;

	gtk_init(&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "RDesktop-GTK");
	gtk_widget_set_usize(GTK_WIDGET(window), 300, 180);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(death), "WM destroy");
	gtk_widget_realize(window);

	table = gtk_table_new(3, 5, FALSE);
	gtk_container_add(GTK_CONTAINER(window), table);
	gtk_widget_show(table);

	label = gtk_label_new("Remote Host:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(label);

	hostname = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), hostname, 1, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(hostname);

	label = gtk_label_new("Username:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(label);

	username = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), username, 1, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(username);

	label = gtk_label_new("Password:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(label);

	password = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), password, 1, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 5);
	gtk_entry_set_visibility(GTK_ENTRY(password), FALSE);
	gtk_widget_show(password);

	label = gtk_label_new("Resolution:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(label);

	resolution_choices = NULL;
	resolution_choices = g_list_append(resolution_choices, "1600x1200");
	resolution_choices = g_list_append(resolution_choices, "1280x1024");
	resolution_choices = g_list_append(resolution_choices, "1024x768");
	resolution_choices = g_list_append(resolution_choices, "800x600");
	resolution_choices = g_list_append(resolution_choices, "640x480");
	resolution = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(resolution), resolution_choices);
	gtk_table_attach(GTK_TABLE(table), resolution, 1, 3, 3, 4, GTK_FILL, GTK_FILL, 0, 5);
	gtk_widget_show(resolution);

	button = gtk_button_new_with_label("Connect");
	gtk_table_attach(GTK_TABLE(table), button, 1, 2, 4, 5, GTK_FILL, GTK_FILL, 0, 5);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(callback), (gpointer) "z0rz");
	gtk_widget_show(button);

	gtk_widget_show(window);

	gtk_timeout_add(100, (GtkFunction) load_settings, (gpointer) 0);

	gtk_main();

	return 0;
}
